`timescale 1ns / 100ps

module align_top (
	input		clk,
	input		rst_n,
	input 		tx_locked,
	input 		rx_locked,
	input [7:0] rx_data,
	input 		pb_align_p,
	
	output reg		sync_valid,
	output reg 	align_bit
	

);

reg [3:0]	cnt_start;	//Count for 1111 and start the first LVDS align, if there isn't align bit from PIO.
wire 		cnt_start_bit; //Use to generate start pulse from count
wire 		start_bit; 	//Start LVDS sync process 
reg 		sync_bit; //Sync successful from align_lvds module
reg 		sync_flag;

reg [2:0] flag;
reg [2:0] cnt;
wire 		sync_bit_p;





always @ ( posedge clk or negedge rst_n )
begin
	if ( !rst_n )
	begin
		cnt_start <= #2 4'b000;
	end
	else if (( tx_locked == 1'b0 ) || ( rx_locked == 1'b0 ) ||( pb_align_p == 1'b1 ))
	begin
		cnt_start <= #2 4'b000;
	end
	else if ( cnt_start == 4'b1111 )
	;
	else
	begin
		cnt_start <= #2 cnt_start + 1'b1;
	end
end	

assign cnt_start_bit = &cnt_start[3:0];

//Generate sync_flag, if rx_data = 8'hf0
always @ ( posedge clk or negedge rst_n )
begin
	if ( !rst_n )
	begin
		sync_flag <= #2 1'b0;
	end
	else if (( tx_locked == 1'b0 ) || ( rx_locked == 1'b0 ) ||( pb_align_p == 1'b1 ))
	begin
		sync_flag <= #2 1'b0;
	end
	else if (( cnt_start == 4'b1111 ) && ( rx_data == 8'hf0 ))
	begin
		sync_flag <= #2 1'b1;
	end
	else
	begin
		sync_flag <= #2 1'b0;
	end
end	

//Generate the pulse from cnt
	rise_edge cnt_rise(
			.clk		( clk ),
			.rst_n		( rst_n ),
			.din		( cnt_start_bit ),
			.dout		( start_bit )
	);


//Start to check the received data pattern.
always @ ( posedge clk or negedge rst_n )
begin
	if ( !rst_n )
	begin
		flag <= #2 3'b000;
	end
	else if ( start_bit == 1'b1 )
	begin
		case (rx_data[7:0])
		8'b11110000: flag <= #2 3'b000;
		8'b01111000: flag <= #2 3'b111;
		8'b00111100: flag <= #2 3'b110;
		8'b00011110: flag <= #2 3'b101;
		8'b00001111: flag <= #2 3'b100;
		8'b10000111: flag <= #2 3'b011;
		8'b11000011: flag <= #2 3'b010;
		8'b11100001: flag <= #2 3'b001;
		default: flag <= #2 3'b000;
	   endcase
	end
	else
	;
end

always @ ( posedge clk or negedge rst_n )
begin
	if ( !rst_n )
	begin
		align_bit	 <= #2 1'b0;
		cnt[2:0] 	<= #2 3'b0;
		sync_bit 	<= #2 1'b0;
	end
	else 
	begin
		if (( flag == 3'b000 ) || ( start_bit == 1'b1 ))
		begin
			align_bit <= #2 1'b0;
			cnt[2:0] <= #2 3'b000;
			sync_bit 	<= #2 1'b0;
		end
		else if ( cnt < flag )
		begin
			if ( align_bit == 1'b1 ) 
			begin
				align_bit <= #2 1'b0;
				cnt <= #2 cnt;	
				sync_bit 	<= #2 1'b0;
			end
			else
			begin
				align_bit <= #2 1'b1;
				cnt <= #2 cnt + 1'b1;	
				sync_bit 	<= #2 1'b0;
			end
		end
		else 
		begin
			align_bit <= #2 1'b0;
			cnt <= #2 cnt;
			sync_bit 	<= #2 1'b1;
		end
	end
end

//Generate the pulse from cnt
	rise_edge sync_rise(
			.clk		( clk ),
			.rst_n		( rst_n ),
			.din		( sync_bit ),
			.dout		( sync_bit_p )
	);
	
//Generate Sync valid signal
always @ ( posedge clk or negedge rst_n )
begin
	if ( !rst_n )
	begin
		sync_valid <= #2 1'b0;
	end
	else if (( tx_locked == 1'b0 ) || ( rx_locked == 1'b0 ) ||( pb_align_p == 1'b1 ))
	begin
		sync_valid <= #2 1'b0;
	end
	else if (( sync_bit_p == 1'b1 ) || ( sync_flag == 1'b1 ))
	begin
		sync_valid <= #2 1'b1;
	end
	else
	;
end	

	
endmodule
